with Integer_Indexed_IO.Debug;
with Ada.Text_IO;

procedure Test_Interactively is

   File : Integer_Indexed_IO.File_Type;

   Line : String (1 .. 80);
   Last : Natural;

   use Ada.Text_IO;
   use Integer_Indexed_IO;


   procedure Read_Item is

      Item, Key : Integer;

   begin

      <<Get_Key>> Put ("Key: ");  Get_Line (Line, Last);

      if Last = 0 then
         Put_Line ("no key specified; try again");
         goto Get_Key;
      end if;

      begin
         Key := Integer'Value (Line (1 .. Last));
      exception
         when Constraint_Error =>
            Put_Line ("bad key");
            goto Get_Key;
      end;

      begin
         Read (File, Key, Item);
         Put_Line ("read item is " & Integer'Image (Item));
      exception
         when Item_Not_Found =>
            Put_Line ("item not found");
      end;

      New_Line;

   end Read_Item;


   procedure Put_Partial_File is

      Nothing_To_Put : Boolean := True;

      procedure Put_Item
        (Item : in     Integer;
         Done : in out Boolean) is
      begin
         Put (Integer'Image (Item));
         Nothing_To_Put := False;
      end;

      procedure Put_Items is
        new Select_Items_Starting_At_Key (Put_Item);

      Key : Integer;

   begin

      <<Get_Key>> Put ("Key: "); Get_Line (Line, Last);

      if Last = 0 then
         Put_Line ("no key specified; try again");
         goto Get_Key;
      end if;

      begin
         Key := Integer'Value (Line (1 .. Last));
      exception
         when Constraint_Error =>
            Put_Line ("bad key");
            goto Get_Key;
      end;


      Put_Items (File, Key);

      if Nothing_To_Put then
         Put ("<no items to display>");
      end if;

      New_Line;

   end Put_Partial_File;



   I : Integer;

   use Debug;

begin

   Create (File); --, Name => "testfile.dat");
   Put_Tree (File);
   New_Line;

   <<Get_Insertion_Value>> Put ("Insert: "); Get_Line (Line, Last);

   if Last = 0 then

      goto Get_Deletion_Value;

   elsif Line (1) = 'p' then

      Put_Tree (File);
      New_Line;
      goto Get_Insertion_Value;

   elsif Line (1) = 'P' then

      Put (File);
      New_Line;
      goto Get_Insertion_Value;

   elsif Line (1) = 'q' then

      Put_Partial_File;
      New_Line;
      goto Get_Insertion_Value;

   elsif Line (1) = 'r' then

      Read_Item;
      goto Get_Insertion_Value;

   elsif Line (1) = 'x' then

      goto Exit_Program;

   else

      begin
         I := Integer'Value (Line (1 .. Last));
      exception
         when Constraint_Error =>
            Put_Line ("bad insertion value");
            goto Get_Insertion_Value;
      end;

      begin
         Write (File, I);
      exception
         when Constraint_Error =>
            Put_Line ("CE during Write");
      end;

      begin
         Put_Tree (File);
      exception
         when Constraint_Error =>
            Put_Line ("CE during Put_Tree");
      end;

      New_Line;
      goto Get_Insertion_Value;

   end if;



   <<Get_Deletion_Value>> Put ("Delete: "); Get_Line (Line, Last);

   if Last = 0 then

      goto Get_Insertion_Value;

   elsif Line (1) = 'r' then

      Read_Item;
      goto Get_Deletion_Value;

   elsif Line (1) = 'p' then

      Put_Tree (File);
      New_Line;
      goto Get_Deletion_Value;

   elsif Line (1) = 'P' then

      Put (File);
      New_Line;
      goto Get_Deletion_Value;

   elsif Line (1) = 'q' then

      Put_Partial_File;
      New_Line;
      goto Get_Deletion_Value;

   elsif Line (1) = 'x' then

      goto Exit_Program;

   else

      begin
         I := Integer'Value (Line (1 .. Last));
      exception
         when Constraint_Error =>
            Put_Line ("bad deletion value");
            goto Get_Deletion_Value;
      end;


      begin
         Remove (File, I);
      exception
         when Constraint_Error =>
            Put_Line ("CE during Remove");
      end;

      begin
         Put_Tree (File);
      exception
         when Constraint_Error =>
            Put_Line ("CE during Put_Tree");
      end;

      New_Line;
      goto Get_Deletion_Value;

   end if;


   <<Exit_Program>> Close (File);

end Test_Interactively;
